@@ -58,6 +58,7 @@
 #define PAM_SM_SESSION
 
 #include <security/pam_modules.h>
+#include <security/pam_modutil.h>
 #include <security/_pam_macros.h>
 #include <security/pam_ext.h>
 
@@ -67,7 +68,7 @@ replace_and_print (pam_handle_t *pamh, const char *mesg)
   char *output;
   size_t length = strlen (mesg) + PAM_MAX_MSG_SIZE;
   char myhostname[HOST_NAME_MAX+1];
-  const void *str;
+  const void *str = NULL;
   const char *p, *q;
   int item;
   size_t len;
@@ -118,7 +119,10 @@ replace_and_print (pam_handle_t *pamh, const char *mesg)
 	    str = &myhostname;
 	}
       else
-	pam_get_item (pamh, item, &str);
+	{
+	  if (pam_get_item (pamh, item, &str) != PAM_SUCCESS)
+	    str = NULL;
+	}
       if (str == NULL)
 	str = "(null)";
       for (q = str; *q != '\0' && len < length - 1; ++q)
@@ -176,16 +180,23 @@ pam_echo (pam_handle_t *pamh, int flags, int argc, const char **argv)
 
       /* load file into message buffer. */
       if ((fstat (fd, &st) < 0) || !st.st_size)
-	return PAM_IGNORE;
+	{
+	  close (fd);
+	  return PAM_IGNORE;
+	}
 
       mtmp = malloc (st.st_size + 1);
       if (!mtmp)
-	return PAM_BUF_ERR;
+	{
+	  close (fd);
+	  return PAM_BUF_ERR;
+	}
 
-      if (read (fd, mtmp, st.st_size) == -1)
+      if (pam_modutil_read (fd, mtmp, st.st_size) == -1)
 	{
 	  pam_syslog (pamh, LOG_ERR, "Error while reading %s: %m", file);
 	  free (mtmp);
+	  close (fd);
 	  return PAM_IGNORE;
 	}
 
