@@ -16,13 +16,13 @@
  * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior
  *    written permission.
- * 
+ *
  * ALTERNATIVELY, this product may be distributed under the terms of
  * the GNU Public License, in which case the provisions of the GPL are
  * required INSTEAD OF the above restrictions.  (This clause is
  * necessary due to a potential bad interaction between the GPL and
  * the restrictions contained in a BSD-style copyright.)
- * 
+ *
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
  * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
  * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -49,7 +49,11 @@
 
 /* indicate the following groups are defined */
 
-#define PAM_SM_SESSION
+#ifdef PAM_STATIC
+# include "pam_unix_static.h"
+#else
+# define PAM_SM_SESSION
+#endif
 
 #include <security/_pam_macros.h>
 #include <security/pam_modules.h>
@@ -63,8 +67,8 @@
  * session module.
  */
 
-PAM_EXTERN int pam_sm_open_session(pam_handle_t * pamh, int flags,
-				   int argc, const char **argv)
+int
+pam_sm_open_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
 	char *user_name, *service;
 	unsigned int ctrl;
@@ -92,14 +96,15 @@ PAM_EXTERN int pam_sm_open_session(pam_handle_t * pamh, int flags,
 	if (login_name == NULL) {
 	    login_name = "";
 	}
-	pam_syslog(pamh, LOG_INFO, "session opened for user %s by %s(uid=%lu)",
-		 user_name, login_name, (unsigned long)getuid());
+	if (off (UNIX_QUIET, ctrl))
+	  pam_syslog(pamh, LOG_INFO, "session opened for user %s by %s(uid=%lu)",
+		     user_name, login_name, (unsigned long)getuid());
 
 	return PAM_SUCCESS;
 }
 
-PAM_EXTERN int pam_sm_close_session(pam_handle_t * pamh, int flags,
-				    int argc, const char **argv)
+int
+pam_sm_close_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
 	char *user_name, *service;
 	unsigned int ctrl;
@@ -122,22 +127,9 @@ PAM_EXTERN int pam_sm_close_session(pam_handle_t * pamh, int flags,
 		         "close_session - error recovering service");
 		return PAM_SESSION_ERR;
 	}
-	pam_syslog(pamh, LOG_INFO, "session closed for user %s",
-		user_name);
+	if (off (UNIX_QUIET, ctrl))
+	  pam_syslog(pamh, LOG_INFO, "session closed for user %s",
+		     user_name);
 
 	return PAM_SUCCESS;
 }
-
-/* static module data */
-#ifdef PAM_STATIC
-struct pam_module _pam_unix_session_modstruct = {
-    "pam_unix_session",
-    NULL,
-    NULL,
-    NULL,
-    pam_sm_open_session,
-    pam_sm_close_session,
-    NULL,
-};
-#endif
-
